/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        18.12.2011
 Description:    DMX_3Kanal_Dimmer


 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdio.h>

#define F_CPU 12000000


#define SSR1_PIN_OUT DDRD |= (1<<PD7);
#define SSR2_PIN_OUT DDRD |= (1<<PD4);
#define SSR3_PIN_OUT DDRD |= (1<<PD2);

#define SSR1_ON		PORTD |= (1<<PD7);
#define SSR1_OFF	PORTD &= ~(1<<PD7);
#define SSR2_ON		PORTD |= (1<<PD4);
#define SSR2_OFF	PORTD &= ~(1<<PD4);
#define SSR3_ON		PORTD |= (1<<PD2);
#define SSR3_OFF	PORTD &= ~(1<<PD2);


#define DMX_BAUD 250000
#define DMX_LOST_TIMEOUT 8000

volatile unsigned char dmx_buffer[514];
volatile unsigned int dmx_lost = DMX_LOST_TIMEOUT;

volatile unsigned int dmx_adresse = 0;
volatile unsigned char phase_off_count = 0;

volatile unsigned char val1;
volatile unsigned char val2;
volatile unsigned char val3;

//############################################################################
//DMX Senderoutine
ISR (USART_RX_vect)
//############################################################################
{
	static unsigned int dmx_channel_rx_count = 0;
	static unsigned char dmx_valid = 0;
	unsigned char tmp = 0;
	
	tmp =  UDR0;
	
	if(UCSR0A&(1<<FE0))
	{
		if(dmx_channel_rx_count > 1)
		{
			dmx_lost = 0;
		}
		dmx_channel_rx_count = 0;	
		dmx_buffer[0] = tmp;
		if(dmx_buffer[0] == 0)
		{
			dmx_valid = 1;
			dmx_channel_rx_count++;
		}
		else
		{
			dmx_valid = 0;
		}
		return;
	}
	
	if(dmx_valid)
	{
		dmx_buffer[dmx_channel_rx_count] = tmp;	
		if(dmx_channel_rx_count < 514)
		{
			dmx_channel_rx_count++;
		}
		return;
	}
}

//############################################################################
//Hier wird die Zeit gezhlt (Tick 1ms)
ISR (TIMER2_COMPA_vect)
//############################################################################
{
	phase_off_count++;
	
	if(phase_off_count>val1) SSR1_ON;
	if(phase_off_count>val2) SSR2_ON;
	if(phase_off_count>val3) SSR3_ON;

	if(dmx_lost<DMX_LOST_TIMEOUT)
	{
		dmx_lost++;
	}
}

//############################################################################
//Hier wird die Zeit gezhlt (Tick 1ms)
ISR (INT1_vect)
//############################################################################
{
	TCNT2 = 0;
	phase_off_count = 0;
	SSR1_OFF;
	SSR2_OFF;
	SSR3_OFF;
}

//############################################################################
//Hauptprogramm
int main (void) 
//############################################################################
{  
	unsigned int dmx_adresse_tmp;
	
	//INT1 ON PHASE SYNC
	EICRA |= (1<<ISC11|1<<ISC10); //The rising edge of INT1 generates an interrupt request
	EIMSK |= (1<<INT1); //External interrupt request 1 enable

	//Init usart DMX-BUS
	UBRR0   = (F_CPU / (DMX_BAUD * 16L) - 1);
	UCSR0B|=(1 << RXEN0 | 1<< RXCIE0);
	UCSR0C|=(1<<USBS0); //USBS0 2 Stop bits	
	sei();//Globale Interrupts Enable
	
	//Timer2 Timecounter fr DMX Ausfall
	TCCR2A |= (1<<WGM21);
	TCCR2B |= (1<<CS22|1<<CS21);
	TIMSK2 |= (1<<OCIE2A);
	OCR2A = F_CPU/256/8000 - 1; //Tick 1ms
		
	SSR1_PIN_OUT;
	SSR2_PIN_OUT;
	SSR3_PIN_OUT;
	SSR1_OFF;
	SSR2_OFF;
	SSR3_OFF;
	
	PORTB |= 0x3F;
	PORTC |= 0xFF;
	
	//Endlosschleife
	while(1)
	{
		dmx_adresse_tmp = 0;
		
		if(!(PINB&(1<<PB0))) dmx_adresse_tmp |= 0x01;
 		if(!(PINB&(1<<PB1))) dmx_adresse_tmp |= 0x02;
		if(!(PINB&(1<<PB2))) dmx_adresse_tmp |= 0x04;
		if(!(PINB&(1<<PB3))) dmx_adresse_tmp |= 0x08;
		if(!(PINB&(1<<PB4))) dmx_adresse_tmp |= 0x10;
		if(!(PINB&(1<<PB5))) dmx_adresse_tmp |= 0x20;
		
		if(!(PINC&(1<<PC0))) dmx_adresse_tmp |= 0x40;
		if(!(PINC&(1<<PC1))) dmx_adresse_tmp |= 0x80;
		if(!(PINC&(1<<PC2))) dmx_adresse_tmp |= 0x0100;
		
		if(dmx_adresse != dmx_adresse_tmp) dmx_adresse =  dmx_adresse_tmp;
		
		val1 = (255 - dmx_buffer[dmx_adresse])/2.2;
		val2 = (255 - dmx_buffer[dmx_adresse+1])/2.2;
		val3 = (255 - dmx_buffer[dmx_adresse+2])/2.2;
		
		if(dmx_lost==DMX_LOST_TIMEOUT)
		{
			dmx_buffer[dmx_adresse] = 0;
			dmx_buffer[dmx_adresse+1] = 0;
			dmx_buffer[dmx_adresse+2] = 0;
		}
	}
}

